require_relative '../studio_game/player'
require_relative '../studio_game/die'
require_relative '../studio_game/game_turn'
require_relative '../studio_game/treasure_trove'
require 'csv'

module StudioGame

	class Game
	attr_reader :title

	def initialize(title)
		@title = title
		@players =[]
	end

	def add_player(player)
		@players<<player
	end

	def load_players(from_file)
		CSV.foreach(from_file) do |row|	
			player = Player.new(row[0].to_s, row[1].to_i)
			add_player(player)
		end
	end


	def save_high_scores(to_file="high_scores.txt")
		File.open(to_file, "w") do |file|
			file.puts "#{@title} High Scores:"
			@players.sort.each do |player|
				file.puts format_name_and_score(player)
			end
		end
	end

	def play(rounds=5)
		puts "There are #{@players.size} players in the #{@title}"
		
		treasures = TreasureTrove::TREASURES
		puts "\nThere are #{treasures.length} treasures in the game"
		treasures.each do |t|
			puts "A #{t.name} is worth #{t.points} points"
		end

		1.upto(rounds) do |round|
			if block_given?
				break if yield
			end
			puts "\nRound #{round}:"

			@players.each do |p|
				GameTurn.take_turn(p)
				puts p
			end
		end

	end

	def total_points
		@players.reduce(0) { |sum, p| sum + p.points }
	end


	def print_stats

		strong_players, wimpy_players = @players.partition { |p| p.strong? }
		puts "\n#{@title} Statistics:"
		
		puts "\n#{strong_players.length} strong players:"
		strong_players.each do |s|
			format_stats(s)
		end

		puts "\n#{wimpy_players.length} wimpy players:"
		wimpy_players.each do |w|
			format_stats(w)
		end


		puts "\n#{@title} High Scores:"
		@players.sort.each do |p|
			puts format_name_and_score(p)
		end

		@players.sort.each do |p|
			puts "#{p.name}'s point total:"
			p.each_found_treasure do |treasure|
				puts "#{treasure.points} total #{treasure.name} points"
			end
			puts "#{p.points} grand total points"
		end

		puts "Total treasure points scored: #{total_points}"

	end

	def format_name_and_score(player)
		formatted_name = player.name.ljust(20, '.')
		"#{formatted_name} #{player.score}"
	end

	def format_stats(player)
		puts "#{player.name} (#{player.health})"
	end



	end
end